//
//  Copyright (c) 1996-2005 by Autodesk, Inc.
//
//  By using this code, you are agreeing to the terms and conditions of
//  the License Agreement included in the documentation for this code.
//
//  AUTODESK MAKES NO WARRANTIES, EXPRESS OR IMPLIED, AS TO THE CORRECTNESS
//  OF THIS CODE OR ANY DERIVATIVE WORKS WHICH INCORPORATE IT. AUTODESK
//  PROVIDES THE CODE ON AN "AS-IS" BASIS AND EXPLICITLY DISCLAIMS ANY
//  LIABILITY, INCLUDING CONSEQUENTIAL AND INCIDENTAL DAMAGES FOR ERRORS,
//  OMISSIONS, AND OTHER PROBLEMS IN THE CODE.
//
//  Use, duplication, or disclosure by the U.S. Government is subject to
//  restrictions set forth in FAR 52.227-19 (Commercial Computer Software
//  Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) (Rights in Technical
//  Data and Computer Software), as applicable.
//

#ifndef _DWFTK_PUBLISHER_H
#define _DWFTK_PUBLISHER_H


///
///\file        dwf/publisher/Publisher.h
///\brief       This file contains the DWFPublishedObject class declaration.
///

#ifndef DWFTK_READ_ONLY

#include "dwfcore/STL.h"
#include "dwfcore/Iterator.h"
#include "dwfcore/Exception.h"
using namespace DWFCore;

#include "dwf/Toolkit.h"
#include "dwf/package/Property.h"
#include "dwf/package/utility/PropertyContainer.h"
#include "dwf/publisher/EmbeddedFont.h"
#include "dwf/publisher/PublishedObject.h"



namespace DWFToolkit
{

///
///\ingroup         dwfpublish
///
///\interface   DWFPublishable   dwf/publisher/Publisher.h    "dwf/publisher/Publisher.h"
///\brief       Base class for interaction with the DWFPublisher.
///\since       7.0.1
///
///             This class defines the most basic requirements for publishing content.
///
class DWFPublishable
{

public:

    ///
    ///         Destructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    virtual ~DWFPublishable() {;}

    ///
    ///         Returns the MIME type of the publishable data.
    ///
    ///\return  The MIME type.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual const DWFString& getMIMEType()
        throw( DWFException ) = 0;

    ///
    ///         Returns a stream from which the publishable data can be read.
    ///
    ///\return  The data stream.
    ///         The caller must release the stream with \b DWFCORE_FREE_OBJECT.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual DWFInputStream* getInputStream()
        throw( DWFException ) = 0;

protected:

    ///
    ///         Constructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    DWFPublishable() {;}
};

///
///\ingroup         dwfpublish
///
///\interface   DWFPublishableResource   dwf/publisher/Publisher.h    "dwf/publisher/Publisher.h"
///\brief       Content that can be added to/associated with a section.
///\since       7.0.1
///
class DWFPublishableResource : public DWFToolkitMemory
                             , public DWFPublishable
{

public:

    ///
    ///\brief   This type defines a list of DWFPublishableResource pointers.
    ///
    typedef vector<DWFPublishableResource*> tList;

public:

    ///
    ///         Destructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    virtual ~DWFPublishableResource() {;}

protected:

    ///
    ///         Constructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    DWFPublishableResource() {;}
};


//
// fwd decl
//
class DWFPublisher;



///
///\ingroup         dwfpublish
///
///\interface   DWFPublishableSection   dwf/publisher/Publisher.h    "dwf/publisher/Publisher.h"
///\brief       Base class for sections that can be used with the DWFPublisher.
///\since       7.0.1
///
class DWFPublishableSection : public DWFPublishable
{

public:

    ///
    ///         Destructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    virtual ~DWFPublishableSection() {;}

    ///
    ///         Write the section structure to \a rPublisher.
    ///
    ///\param   rPublisher      The publisher that captures the output of the section.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void publish( DWFPublisher& rPublisher )
        throw( DWFException ) = 0;

    ///
    ///         Request the provided font be embedded in the section.
    /// 
    ///\param   pFont       The font data object to embed.  This object will assume ownership of \a pFont 
    ///                     and will delete it with \b DWFCORE_FREE_OBJECT.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void embedFont( DWFEmbeddedFont* pFont )
        throw( DWFException )
    {
        (void)pFont;
        _DWFCORE_THROW( DWFNotImplementedException, L"Font embedding not supported by this object" );
    }

    ///
    ///         Request that the resource be added to the section.
    ///
    ///         Use this method for adding thumbnails, previews, etc.
    ///
    ///\param   pResource   The resource to add. This object will assume ownership of \a pResource 
    ///                     and will delete it with \b DWFCORE_FREE_OBJECT.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void addResource( DWFPublishableResource* pResource )
        throw( DWFException )
    {
        (void)pResource;
        _DWFCORE_THROW( DWFNotImplementedException, L"Additional resources are not supported by this object" );
    }

    ///
    ///         Returns the list of resources added to this section.
    ///
    ///\return  A pointer to an iterator than will provide the resources.
    ///         This pointer must be deleted by the caller with \b DWFCORE_FREE_OBJECT
    ///         but the resource pointers themselves should not be.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual DWFIterator<DWFPublishableResource*>* getResources()
        throw( DWFException )
    {
        _DWFCORE_THROW( DWFNotImplementedException, L"Additional resources are not supported by this object" );
    }

protected:

    ///
    ///         Constructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    DWFPublishableSection() {;}
};

///
///\ingroup         dwfpublish
///
///\class       DWFPropertyVisitor   dwf/publisher/Publisher.h    "dwf/publisher/Publisher.h"
///\brief       Base implementation and interface defintion for objects (visitors) that
///             publishers can use with publishable sources to extract their property data.
///\since       7.0.1
///
class DWFPropertyVisitor
{

public:

    ///
    ///         Destructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    virtual ~DWFPropertyVisitor()
        throw()
    {;}

    ///
    ///         Default visitation implementation.  Specializations should
    ///         invoke this base method to ensure any composed visitors are called.
    ///
    ///\param   rProperty   A property from the publishable source.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void visitProperty( DWFProperty& rProperty )
        throw( DWFException )
    {
        if (_pComposite)
        {
            _pComposite->visitProperty( rProperty );
        }
    }

    ///
    ///         Default visitation implementation.  Specializations should
    ///         invoke this base method to ensure any composed visitors are called.
    ///
    ///\param   rPropertyContainer   The property container from the publishable source.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void visitPropertyContainer( DWFPropertyContainer& rPropertyContainer )
        throw( DWFException )
    {
        if (_pComposite)
        {
            _pComposite->visitPropertyContainer( rPropertyContainer );
        }
    }

protected:

    ///
    ///         Constructor
    ///
    ///\param   pComposite      A pointer a chained visitor.
    ///                         This pointer will not be deleted.
    ///\throw   None
    ///
    _DWFTK_API
    DWFPropertyVisitor( DWFPropertyVisitor* pComposite = NULL )
        throw()
        : _pComposite( pComposite )
    {;}

private:

    DWFPropertyVisitor( const DWFPropertyVisitor& );
    DWFPropertyVisitor& operator=( const DWFPropertyVisitor& );

private:

    DWFPropertyVisitor* _pComposite;
};

///
///\ingroup         dwfpublish
///
///\class       DWFEmbeddedFontVisitor   dwf/publisher/Publisher.h    "dwf/publisher/Publisher.h"
///\brief       Base implementation and interface defintion for objects (visitors) that
///             publishers can use with publishable sources to extract their embedded font data.
///\since       7.0.1
///
class DWFEmbeddedFontVisitor
{

public:

    ///
    ///         Destructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    virtual ~DWFEmbeddedFontVisitor()
        throw()
    {;}

    ///
    ///         Default visitation implementation.  Specializations should
    ///         invoke this base method to ensure any composed visitors are called.
    ///
    ///\param   rFont       An embedded font from the publishable source.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void visitEmbeddedFont( const DWFEmbeddedFont& rFont )
        throw( DWFException )
    {
        if (_pComposite)
        {
            _pComposite->visitEmbeddedFont( rFont );
        }
    }

protected:

    ///
    ///         Constructor
    ///
    ///\param   pComposite      A pointer a chained visitor.
    ///                         This pointer will not be deleted.
    ///\throw   None
    ///
    _DWFTK_API
    DWFEmbeddedFontVisitor( DWFEmbeddedFontVisitor* pComposite = NULL )
        throw()
        : _pComposite( pComposite )
    {;}

private:

    DWFEmbeddedFontVisitor( const DWFEmbeddedFontVisitor& );
    DWFEmbeddedFontVisitor& operator=( const DWFEmbeddedFontVisitor& );

private:

    DWFEmbeddedFontVisitor* _pComposite;
};


///
///\ingroup         dwfpublish
///
///\class       DWFPublisher   dwf/publisher/Publisher.h    "dwf/publisher/Publisher.h"
///\brief       Base implementation and interface defintion for objects (publishers) that
///             can process and publish known content.
///\since       7.0.1
///
///             Publisher objects are used to extract data from publishable structural constructs (Model, etc.)
///             A publisher can be configured with strongly type visitors that will be called upon
///             as the structure is traversed.  Complex and compound visitors can be created
///             for additional functionality.  For example, a compound DWFPublishedObject visitor could be used
///             to insert a record in a database for every "object" in the structure. 
///             Another is as a base class for a aggregate object that additionally
///             implements whatever visitor interfaces it needs.  This solution is preferrable if common data
///             must be shared across visitors.  
///
///             The basic publishing workflow is as follows:
///             -# Create an instance of a [derived type] DWFPublisher (DWFPackagePublisher is an example implementation.)
///             -# Create an instance of a [derived type] DWFPublishableSection (DWFModel is an example implementation.)
///             -# Add content (DWFPublishable types) to the section.
///             -# Pass the publisher to the \a publish() method of the section.  The publisher will process the section
///                content according to it's implementation details.
///             -# Invoke the \a publish() method on the publisher.  This will complete the process.
///
///             For a more advanced workflow, repeat steps 2, 3 and 4 above to add multiple sections to the final publication.
///
///\note        It should be obvious but there is nothing requiring a DWFPublisher to actually produce a DWF pacakge file.
///             A reference implementation (DWFPackagePublisher) is provided with the toolkit.
///
class DWFPublisher
{

public:

    ///
    ///         Constructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    DWFPublisher()
        throw()
        : _pPublishedObjectVisitor( NULL )
        , _pPropertyVisitor( NULL )
        , _pEmbeddedFontVisitor( NULL )
    {;}

    ///
    ///         Destructor
    ///
    ///\throw   None
    ///
    _DWFTK_API
    virtual ~DWFPublisher()
        throw()
    {;}

    ///
    ///         Invokes the final publication process.
    ///
    ///\throw   None
    ///
    _DWFTK_API
    virtual void publish()
        throw( DWFException )
    {;}

    ///
    ///         Returns a visitor that can process published objects
    ///         in order to retrieve or characterize their structure.
    ///
    ///\return  A published object visitor.  This pointer must not be deleted by the caller.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual DWFPublishedObject::Visitor* getPublishedObjectVisitor()
        throw( DWFException )
    {
        return _pPublishedObjectVisitor;
    }

    ///
    ///         Sets the visitor that can process published objects
    ///         in order to retrieve or characterize their structure.
    ///
    ///\param   pVisitor    The published object visitor. This pointer will not
    ///                     be deleted by the publisher.
    ///\throw   None
    ///
    _DWFTK_API
    void setPublishedObjectVisitor( DWFPublishedObject::Visitor* pVisitor )
        throw()
    {
        _pPublishedObjectVisitor = pVisitor;
    }

    ///
    ///         Returns a visitor that can process a property and/or property container.
    ///
    ///\return  A property visitor.  This pointer must not be deleted by the caller.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual DWFPropertyVisitor* getPropertyVisitor()
        throw( DWFException )
    {
        return _pPropertyVisitor;
    }

    ///
    ///         Sets the visitor that can process a property and/or property container.
    ///
    ///\param   pVisitor    The property visitor. This pointer will not
    ///                     be deleted by the publisher.
    ///\throw   None
    ///
    _DWFTK_API
    void setPropertyVisitor( DWFPropertyVisitor* pVisitor )
        throw()
    {
        _pPropertyVisitor = pVisitor;
    }

    ///
    ///         Returns a visitor that can process a font.
    ///
    ///\return  A font visitor.  This pointer must not be deleted by the caller.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual DWFEmbeddedFontVisitor* getEmbeddedFontVisitor()
        throw( DWFException )
    {
        return _pEmbeddedFontVisitor;
    }

    ///
    ///         Sets the visitor that can process a font.
    ///
    ///\param   pVisitor    The font visitor. This pointer will not
    ///                     be deleted by the publisher.
    ///\throw   None
    ///
    _DWFTK_API
    void setEmbeddedFontVisitor( DWFEmbeddedFontVisitor* pVisitor )
        throw()
    {
        _pEmbeddedFontVisitor = pVisitor;
    }

    ///
    ///         This callback should be invoked by \a pPublishable
    ///         as soon as it starts it's \a publish() processing
    ///         passing itself back as the parameter.
    ///
    ///\param   pPublishable    The object being published.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void preprocess( DWFPublishable* pPublishable )
        throw( DWFException )
    {
        (void)pPublishable;
    }

    ///
    ///         This callback should be invoked by \a pPublishable
    ///         when it has completed it's \a publish() processing
    ///         passing itself back as the parameter.
    ///
    ///\param   pPublishable    The object being published.
    ///\throw   DWFException
    ///
    _DWFTK_API
    virtual void postprocess( DWFPublishable* pPublishable )
        throw( DWFException )
    {
        (void)pPublishable;
    }

private:

    DWFPublishedObject::Visitor*    _pPublishedObjectVisitor;
    DWFPropertyVisitor*             _pPropertyVisitor;
    DWFEmbeddedFontVisitor*         _pEmbeddedFontVisitor;

private:

    DWFPublisher( const DWFPublisher& );
    DWFPublisher& operator=( const DWFPublisher& );
};


}


#endif  
#endif

